from Acquisition import aq_base, aq_inner, aq_parent
from Globals import InitializeClass
from Products.CMFCore.utils import getToolByName
from Acquisition import aq_chain

#---------------------------- patches for CMFCatalogAware -------------------------------------
#

from Products.CMFCore.CMFCatalogAware import CMFCatalogAware

def manage_afterAdd(self, item, container):
    """ Perform version related operations
    """
    if hasattr(self, '_v_version_bundle_'):
        front_end_id = self.getId()
        for version in self._v_version_bundle_:
            # rename all versions, in case it's a rename operation
            v_id = '%s~v~%s' % (self.getId(), version.version_number)
            version._setId(v_id)
            container._setOb(v_id, version)
            version = container._getOb(v_id)
            version.manage_afterAdd(version, container)
        delattr(self, '_v_version_bundle_')
    self._original_manage_afterAdd(item, container)

def manage_afterClone(self, item):
    """ Remove version infomation
    """
    if self.getId().find('~v~') < 0 and hasattr(self, 'version_cluster_id'):
        delattr(self, 'version_cluster_id')
    self._original_manage_afterClone(item)

def manage_beforeDelete(self, item, container):
    """ Bundle versions under this object
    """
    # due to a bug in CMFCatalogAware.__recurse,
    # the container passed here might not be the real one
    container = aq_parent(aq_inner(self))
    if self.getId().find('~v~') < 0 and hasattr(self, 'version_cluster_id'):
        self._v_version_bundle_ = [] # declared as volatile by intension
        pvs = getToolByName(obj, 'portal_content_versioning_system')
        for id in pvs.getAllVersionIds(self):
            # find out all hidden versions
            if id.find('~v~') > 0 :
                self._v_version_bundle_.append(aq_base(container._getOb(id)))
                container._delObject(id)
    self._original_manage_beforeDelete(item, container)

if not hasattr(CMFCatalogAware, '_original_manage_afterAdd'):
    # test if the originals have been renamed
    # otherwise, the product refresh will point the original to the pathces instead
    CMFCatalogAware._original_manage_afterAdd = CMFCatalogAware.manage_afterAdd
    CMFCatalogAware._original_manage_afterClone = CMFCatalogAware.manage_afterClone
    CMFCatalogAware._original_manage_beforeDelete = CMFCatalogAware.manage_beforeDelete

CMFCatalogAware.manage_afterAdd = manage_afterAdd
CMFCatalogAware.manage_afterClone = manage_afterClone
CMFCatalogAware.manage_beforeDelete = manage_beforeDelete

InitializeClass(CMFCatalogAware)

#---------------------------- patches for skins mechanism -------------------------------------
#

from Products.CMFCore.SkinsContainer import SkinsContainer
from Products.CMFCore.Skinnable import SkinnableObjectManager

# getSkin is the function of interest,
if hasattr(SkinsContainer, 'getSkin'):
    # looks like pre-1.4, and the getSkin is a method of SkinsContainer
    patchee = SkinsContainer

    def getSkin(self, request):
        """ Lay down SkinTrap
        """
        skinob = self._original_getSkin(request)
        if skinob and hasattr(self, '_skin_trap_'):
            self._skin_trap_.backUpSkin(skinob)
            # insert skin trap as the first item in the acquisition path
            skin_trap = getattr(aq_base(self), '_skin_trap_')
            skinob = skin_trap.__of__(skinob)
        return skinob
else:
    # cmf 1.4 has moved it to SkinnableObjectManager
    patchee = SkinnableObjectManager

    def getSkin(self, name=None):
        """ Lay down SkinTrap
        """
        skinob = self._original_getSkin(name)
        self = self.portal_skins
        if skinob and hasattr(self, '_skin_trap_'):
            self._skin_trap_.backUpSkin(skinob)
            # insert skin trap as the first item in the acquisition path
            skin_trap = getattr(aq_base(self), '_skin_trap_')
            skinob = skin_trap.__of__(skinob)
        return skinob

if not hasattr(patchee , '_original_getSkin'):
    patchee._original_getSkin = patchee.getSkin
patchee.getSkin = getSkin

InitializeClass(SkinsContainer)

#---------------------------- patches for FactoryTool if in plone -------------------------------------
#

def isTemporary(self, obj=None, container=None, id=None):
    """ Patch FactoryTool so that it won't mistake a hidden version
        is a temporary object.
    """
    if id is None:
        id = obj.getId()
    if id.find('~v~') > 0:
        return 0
    else:
        self._original_isTemporary(obj, container, id)

try:
    from Products.CMFPlone.FactoryTool import FactoryTool
except:
    # we're not in plone
    FactoryTool = None

if FactoryTool:
    if not hasattr(FactoryTool, '_original_isTemporary'):
        FactoryTool._original_isTemporary = FactoryTool.isTemporary
    FactoryTool.isTemporary = isTemporary

