import Globals, re
from OFS.SimpleItem import SimpleItem
from Products.CMFCore.utils import getToolByName, _checkPermission
from Acquisition import aq_chain, aq_parent, aq_inner, aq_base, Implicit, Explicit, Acquired
from time import time
from AccessControl import getSecurityManager, ClassSecurityInfo

from ZPublisher.mapply import mapply
from ZPublisher.Publish import call_object, missing_name, dont_publish_class

from Products.CMFCore.CMFCorePermissions import AccessContentsInformation, ModifyPortalContent, View, ReviewPortalContent
from Products.CMFCore.WorkflowCore import WorkflowException
from PVSPermissions import CreateVersions

def relayScriptInvocation(new_context, script_name, REQUEST, upper_skin):
    # find the next script along the acquisition path, call it with the new context
    script = upper_skin.aq_acquire(script_name)
    script = script.__of__(new_context)
    return mapply(script, REQUEST.args, REQUEST,
                    call_object,1,
                    missing_name,
                    dont_publish_class,
                    REQUEST, bind=1)

def getIdForNewVersion(obj):
    pvs = getToolByName(obj, 'portal_content_versioning_system')
    ids = pvs.getAllVersionIds(obj)
    prefix = obj.getId()
    if prefix.find('~v~') < 0:
        if hasattr(obj, 'version_number'):
            prefix += '~v~' + obj.version_number
        else:
            prefix += '~v~1'

    child_version_ids = [id for id in ids if re.match(prefix + '.\d+\Z', id)]
    id_suffixes = [int(id.split('.')[-1]) for id in child_version_ids]
    if id_suffixes:
        new_suffix = max(id_suffixes) + 1
    else:
        new_suffix = 1
    return '%s.%d' % (prefix, new_suffix)

def editActionTrap(self, container, REQUEST, upper_skin):
    "Hijack the edit action"

    in_plone = hasattr(self, 'plone_utils')
    if in_plone:
        # plone lets user to edit content id in edit form
        # disallow that if the content is a hidden version
        if self.getId().find('~v~') > 0:
            REQUEST.set('id', self.getId()) # no id changes

    if _checkPermission(CreateVersions, self) \
       and not getattr(aq_base(self), 'isPrincipiaFolderish', 0) \
       and not _checkPermission(ModifyPortalContent, self) :

        # user qualified for making new version
        container = aq_parent(aq_inner(self))
        if not hasattr(aq_base(self), 'version_cluster_id'):
            # this object isn't verionized. shape it up
            self.version_cluster_id = int(time())
            self.version_number = '1'
            self.reindexObject() # expose the object as versionized to the catalog

        new_id = getIdForNewVersion(self)
        new_version = self._getCopy(container)
        new_version._setId(new_id)
        new_version.version_number = new_id.split('~v~')[-1]
        container._setOb(new_id, new_version)
        new_context = container._getOb(new_id)

        if in_plone:
            REQUEST.set('id', new_id) # disallow id changes

        # set ownership
        user = getSecurityManager().getUser()
        if user is not None:
            name = user.getUserName()
            if name != 'Anonymous User':
                new_context.__ac_local_roles__ = {name : ['Owner']}


        new_context.notifyWorkflowCreated() # set new version into the default state
    else:
        new_context = self

    return relayScriptInvocation(new_context, self.script_name, REQUEST, upper_skin)

def contentStatusModify(self, container, REQUEST, upper_skin):
    "Hijack the publish workflow action"

    if self.getId().find('~v~') > 0 \
       and _checkPermission(ReviewPortalContent, self) \
       and REQUEST.workflow_action == 'publish' :

        to_be_published = aq_base(self)
        container._delObject(to_be_published.getId())

        front_end_id = to_be_published.getId().split('~v~')[0]
        to_be_archived = aq_base(container._getOb(front_end_id))
        to_be_archived_id = front_end_id + '~v~' + to_be_archived.version_number

        # switch ids
        to_be_archived._setId(to_be_archived_id)
        to_be_published._setId(front_end_id)

        # switch objs
        container._setOb(to_be_archived.getId(), to_be_archived)
        container._setOb(front_end_id, to_be_published)

        the_archived = container._getOb(to_be_archived_id)
        new_context = container._getOb(front_end_id)

        workflow_tool = getToolByName(self, 'portal_workflow')
        # check if the archived item is published
        review_state = workflow_tool.getInfoFor(the_archived, 'review_state')
        if review_state == 'published':
            # try perform 'archive' transition
            try:
                workflow_tool.doActionFor(the_archived, 'archive')
            except WorkflowException:
                # the associated workflow is not versionized
                # set the archived into default state
                the_archived.notifyWorkflowCreated()

        # force to reindex everything on the two objects
        # the workflow methods, such as notifyWorkflowCreated, doActionFor etc.
        # will only reindex workflow related variables
        the_archived.reindexObject()
        new_context.reindexObject()
    else:
        new_context = self

    return relayScriptInvocation(new_context, 'content_status_modify', REQUEST, upper_skin)

class ExternalSkinScript(Explicit, SimpleItem):
    "A wrapper over the mothods above"
    security = ClassSecurityInfo()

    _v_upper_skin = None

    def __init__(self, function):
        self._func = function

    def setUpperSkinData(self, upper_skin):
        self._v_upper_skin = upper_skin

    # at this point, everyone should be able to call the script
    # it's up to the target script to control who can run it
    security.declareProtected(View, '__call__' )
    def __call__(self):
        skinstool = getToolByName(self, 'portal_skins')
        skintrap = skinstool._skin_trap_
        upper_skin = skintrap._v_upper_skin[0]
        REQUEST = self.REQUEST

        context = self.aq_parent.this() # the new context is the cmf obj this script attaches to
        container = aq_parent(aq_inner(context))
        return apply(self._func, (context, container, REQUEST, upper_skin))


class SkinTrap(SimpleItem):
    "A virtual skin layer that will intercepts the skin script acquicition"

    meta_type = 'Skin Trap'
    id = '_skin_trap_'
    _v_upper_skin = None # declare it as class var so that it can be accessed directly, instead of __dict__

    _contentStatusModify = ExternalSkinScript(contentStatusModify)
    _editActionTrap = ExternalSkinScript(editActionTrap)

    def backUpSkin(self, upper_skin):
        if upper_skin:
            self._v_upper_skin = (upper_skin, ) # wrap it in a list so as to keep the acquisition chain

    def __getattr__(self, name):
        " "
	print 'sit', name
        if name == 'content_status_modify':
            return self._contentStatusModify

        upper_skin = self._v_upper_skin[0]
        # only interested in callings to python script which ends with '_edit', not starts with 'validate'
        if hasattr(upper_skin, name) and name.endswith('_edit') and not name.startswith('validate'):
            attr = getattr(upper_skin, name)
            if hasattr(attr, 'meta_type') and attr.meta_type.endswith('Script (Python)'):
                self.script_name = name
                return self._editActionTrap
        raise AttributeError, name

Globals.InitializeClass(SkinTrap)
